//+------------------------------------------------------------------+
//|                                                          ADX.mq5 |
//|                        Copyright 2009, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright   "2009, MetaQuotes Software Corp."
#property link        "http://www.mql5.com"
#property description "Average Directional Movement Index"
#include <MovingAverages.mqh>

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_plots   3
#property indicator_type1   DRAW_LINE
#property indicator_color1  LightSeaGreen
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
#property indicator_type2   DRAW_LINE
#property indicator_color2  YellowGreen
#property indicator_style2  STYLE_DOT
#property indicator_width2  1
#property indicator_type3   DRAW_LINE
#property indicator_color3  Wheat
#property indicator_style3  STYLE_DOT
#property indicator_width3  1
#property indicator_label1  "ADX_TF2"
#property indicator_label2  "+DI"
#property indicator_label3  "-DI"
//--- input parameters
input ENUM_TIMEFRAMES      InpTimeFrame_2=PERIOD_H1;      // Timeframe 2 (TF2) period
input int InpPeriodADX=14; // Period
input ENUM_APPLIED_PRICE   InpAppliedPrice=PRICE_CLOSE;

//---- buffers
double    ExtADXBuffer_TF2[];
double    ExtPDIBuffer_TF2[];
double    ExtNDIBuffer_TF2[];
double    ExtPDBuffer_TF2[];
double    ExtNDBuffer_TF2[];
double    ExtTmpBuffer_TF2[];

double iADXBuffer[];

   
//--- arrays Tdouble iRSIBuffer[];F2 - to retrieve TF 2 values of buffers and/or timeseries
double                     ExtADXArray_TF2[];       // intermediate array to hold TF2 RSI buffer values

//--- global variables
int                        PeriodRatio=1;           // ratio between timeframe 1 (TF1) and timeframe 2 (TF2)
int                        PeriodSeconds_TF1;       // TF1 period in seconds
int                        PeriodSeconds_TF2;       // TF2 period in seconds
int       ExtADXPeriod;
//--- indicator handles TF2
int                        ExtADXHandle_TF2;        // RSI handle TF2
//--- turn on/off error messages
bool                       ShowErrorMessages=false;  // turn on/off error messages for debugging

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
  {
//--- check for input parameters
   if(InpPeriodADX>=100 || InpPeriodADX<=0)
     {
      ExtADXPeriod=14;
      printf("Incorrect value for input variable Period_ADX=%d. Indicator will use value=%d for calculations.",InpPeriodADX,ExtADXPeriod);
     }
   else ExtADXPeriod=InpPeriodADX;
//---- indicator buffers
   SetIndexBuffer(0,ExtADXBuffer_TF2);
   SetIndexBuffer(1,ExtPDIBuffer_TF2);
   SetIndexBuffer(2,ExtNDIBuffer_TF2);
   SetIndexBuffer(3,ExtPDBuffer_TF2,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,ExtNDBuffer_TF2,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,ExtTmpBuffer_TF2,INDICATOR_CALCULATIONS);
//--- set buffers as series, most recent entry at index [0]
   ArraySetAsSeries(ExtADXBuffer_TF2,true);
//--- set arrays as series, most recent entry at index [0]
   ArraySetAsSeries(ExtADXArray_TF2,true);

//--- indicator digits
   IndicatorSetInteger(INDICATOR_DIGITS,2);
//--- set draw begin
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,ExtADXPeriod<<1);
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,ExtADXPeriod);
   PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,ExtADXPeriod);
//--- indicator short name
   string short_name="ADX("+string(ExtADXPeriod)+")";
   IndicatorSetString(INDICATOR_SHORTNAME,short_name);
//--- change 1-st index label
   PlotIndexSetString(0,PLOT_LABEL,short_name);
//--- calculate at which bar to start drawing indicators
   PeriodSeconds_TF1=PeriodSeconds();
   PeriodSeconds_TF2=PeriodSeconds(InpTimeFrame_2);

   if(PeriodSeconds_TF1<PeriodSeconds_TF2)
      PeriodRatio=PeriodSeconds_TF2/PeriodSeconds_TF1;

//--- get RSI handle
  ExtADXHandle_TF2=iADX(NULL,InpTimeFrame_2,InpPeriodADX,InpAppliedPrice);
  


//---- end of initialization function
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &Time[],
                const double &Open[],
                const double &High[],
                const double &Low[],
                const double &Close[],
                const long &TickVolume[],
                const long &Volume[],
                const int &Spread[])
  {
//--- set arrays as series, most recent entry at index [0]
   ArraySetAsSeries(Time,true);

 //--- check for data
   int bars_TF2=Bars(NULL,InpTimeFrame_2);
   if(bars_TF2<InpPeriodADX)
      return(0);

//--- not all data may be calculated
   int calculated_TF2;
   
   calculated_TF2=BarsCalculated(ExtADXHandle_TF2);
   if(calculated_TF2<bars_TF2)
     {
      if(ShowErrorMessages) Print("Not all data of ExtRsiHandle_TF2 has been calculated (",calculated_TF2," bars). Error",GetLastError());
      return(0);
     } 
  //--- set limit for which bars need to be (re)calculated
   int limit;
   if(prev_calculated==0 || prev_calculated<0 || prev_calculated>rates_total)
      limit=rates_total-1;
   else
      limit=rates_total-prev_calculated;

//--- create variable required to convert between TF1 and TF2
   datetime convertedTime;

//--- loop through TF1 bars to set buffer TF1 values
   for(int i=limit;i>=0;i--)
     {
      //--- convert time TF1 to nearest earlier time TF2 for a bar opened on TF2 which is to close during the current TF1 bar
      //--- use this for calculations with PRICE_CLOSE, PRICE_HIGH, PRICE_LOW, PRICE_MEDIAN, PRICE_TYPICAL, PRICE_WEIGHTED
      if(InpAppliedPrice!=PRICE_OPEN)
         convertedTime=Time[i]+PeriodSeconds_TF1-PeriodSeconds_TF2;
      //--- convert time TF1 to nearest earlier time TF2 for a bar opened on TF2 at the same time or before the current TF1 bar
      //--- use this for calculations with PRICE_OPEN
      if(InpAppliedPrice==PRICE_OPEN)
         convertedTime=Time[i];

      //--- check if TF2 data is available at convertedTime
      datetime tempTimeArray_TF2[];
      CopyTime(NULL,InpTimeFrame_2,calculated_TF2-1,1,tempTimeArray_TF2);
      //--- no TF2 data available
      if(convertedTime<tempTimeArray_TF2[0])
        {
         ExtADXBuffer_TF2[i]=EMPTY_VALUE;
         continue;
        }

      //--- get rsi buffer values of TF2
      if(CopyBuffer(ExtADXHandle_TF2,0,convertedTime,1,ExtADXArray_TF2)<=0)
        {
         if(ShowErrorMessages) Print("Getting ADX TF2 failed! Error",GetLastError());
         return(0);
        }
      //--- set rsi TF2 buffer on TF1
      else
         ExtADXBuffer_TF2[i]=ExtADXArray_TF2[0];
     }

//--- checking for bars count
   if(rates_total<ExtADXPeriod)
      return(0);
//--- detect start position
   int start;
   if(prev_calculated>1) start=prev_calculated-1;
   else
     {
      start=1;
      ExtPDIBuffer_TF2[0]=0.0;
      ExtNDIBuffer_TF2[0]=0.0;
      ExtADXBuffer_TF2[0]=0.0;
     }
//--- main cycle
   for(int i=start;i<rates_total && !IsStopped();i++)
     {
      //--- get some data
      double Hi    =High[i];
      double prevHi=High[i-1];
      double Lo    =Low[i];
      double prevLo=Low[i-1];
      double prevCl=Close[i-1];
      //--- fill main positive and main negative buffers
      double dTmpP=Hi-prevHi;
      double dTmpN=prevLo-Lo;
      if(dTmpP<0.0)   dTmpP=0.0;
      if(dTmpN<0.0)   dTmpN=0.0;
      if(dTmpP>dTmpN) dTmpN=0.0;
      else
        {
         if(dTmpP<dTmpN) dTmpP=0.0;
         else
           {
            dTmpP=0.0;
            dTmpN=0.0;
           }
        }
      //--- define TR
      double tr=MathMax(MathMax(MathAbs(Hi-Lo),MathAbs(Hi-prevCl)),MathAbs(Lo-prevCl));
      //---
      if(tr!=0.0)
        {
         ExtPDBuffer_TF2[i]=100.0*dTmpP/tr;
         ExtNDBuffer_TF2[i]=100.0*dTmpN/tr;
        }
      else
        {
         ExtPDBuffer_TF2[i]=0.0;
         ExtNDBuffer_TF2[i]=0.0;
        }
      //--- fill smoothed positive and negative buffers
      ExtPDIBuffer_TF2[i]=ExponentialMA(i,ExtADXPeriod,ExtPDIBuffer_TF2[i-1],ExtPDBuffer_TF2);
      ExtNDIBuffer_TF2[i]=ExponentialMA(i,ExtADXPeriod,ExtNDIBuffer_TF2[i-1],ExtNDBuffer_TF2);
      //--- fill ADXTmp buffer
      double dTmp=ExtPDIBuffer_TF2[i]+ExtNDIBuffer_TF2[i];
      if(dTmp!=0.0)
         dTmp=100.0*MathAbs((ExtPDIBuffer_TF2[i]-ExtNDIBuffer_TF2[i])/dTmp);
      else
         dTmp=0.0;
      ExtTmpBuffer_TF2[i]=dTmp;
      //--- fill smoothed ADX buffer
      ExtADXBuffer_TF2[i]=ExponentialMA(i,ExtADXPeriod,ExtADXBuffer_TF2[i-1],ExtTmpBuffer_TF2);
     }
//---- OnCalculate done. Return new prev_calculated.
   return(rates_total);
  }
//+------------------------------------------------------------------+
